/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * tad_app.h
 * Copyright (C) 2014 WSID <jongsome@naver.com>
 * 
 * touchscreen-autodisabler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * touchscreen-autodisabler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.";
 */

#ifndef __TAD_APP_HEADER
#define __TAD_APP_HEADER

#include <stddef.h>
#include <stdlib.h>
#include <stdbool.h>
#include <inttypes.h>

#include <getopt.h>

#include "tad_def.h"

/* enum declarations **********************************************************/

/*
 * TadOptionType:
 * @OPTION_VOID: option does not accept any options. represented by #bool.
 * @OPTION_BOOL: boolean option. represented by #bool.
 * @OPTION_STRING: string option. represented by #char*
 * @OPTION_ENUN: an (psuedo) enumeration option. represented by #int
 * 
 * The type for options.
 * 
 * for %OPTION_ENUM, null-terminating array of strings should be in
 * #TadOptionItem.type.
 * 
 * for example,
 * 
 * TadOptionItem a item = {
 *	  OPTION_ENUM,
 *    {"asdf", "qwer", NULL},
 *    "asdf"
 * };
 *
 */
typedef enum _TadOptionType {
	OPTION_VOID,
	OPTION_BOOL,
	OPTION_STRING,
	OPTION_ENUM
} TadOptionType;


/*
 * TadOptionID:
 * @OPTION_ZERO: dummy placeholder for invaild 0 value.
 * @OPTION_NONE: the option passed with nothing.
 * @OPTION_HELP: this option is 'help'
 * @OPTION_VERSION: this options is 'version'
 * @OPTION_WATCH: this option is 'watch'
 * @OPTION_CONTROL: this option is 'control'
 * @OPTION_CLEAR_TO_ENABLE: this options is 'clear to enable'
 * @OPTION_DAEMONIZE: this option is 'daemonize'
 * 
 * id or shortened name of options.
 */
typedef enum _TadOptionID {
	OPTIONS_ZERO,
	OPTIONS_NONE,
	OPTIONS_HELP = 'h',
	OPTIONS_VERSION = 'v',
	OPTIONS_WATCH = 'w',
	OPTIONS_CONTROL = 'c',
	OPTIONS_CLEAR_TO_ENABLE = 'e',
	OPTIONS_DAEMONIZE = 'd',
	OPTIONS_PRINT = 'p'
} TadOptionID;



/* struct declarations ********************************************************/

typedef struct _TadOptionItem {
	TadOptionType   type;
	void*			type_data;
	size_t			offset;
} TadOptionItem;

typedef struct _TadOption {
	// touchscreen_autodisabelr
	char*			watch;
	char*			control;

	// tad_app
	bool			help;
	bool			version;
	bool			daemonize;
	int				print;
	
	// tad_state
	bool			clear_to_enable;
	
} TadOption;

/* callback type **************************************************************/

typedef void	(*TadToggleFunc)			(bool			state,
				                             void*			user_data);

/* constant declaration *******************************************************/

extern const struct option long_opts[];

extern const char* opt_helps[];

extern const TadOptionItem opt_items[];



/* function prototypes - core application functions ***************************/
char*		tad_lock_file_path			(void);

void		tad_lock					(void);
void		tad_unlock					(void);

void		tad_daemonize				(void);


/* function prototype - option processing *************************************/

void		tad_option_set				(int				argc,
							             char**				argv,
							             TadOption*   		result);

char*		tad_option_to_string		(TadOption*			option);

void		tad_option_help				(void);

/* function prototype - state management **************************************/
void			tad_state_set				(TadTransType   trans_type);

TadState		tad_state_get_state			(void);

bool			tad_state_get_touch_enabled (void);

TadToggleFunc   tad_state_on_toggle			(TadToggleFunc  func,
				                             void*			user_data);

void			tad_transit					(TadEvent	stimulus);

#endif  //TAD_APP_HEADER
