/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * tad_device.h
 * Copyright (C) 2014 WSID <jongsome@naver.com>
 * 
 * touchscreen-autodisabler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * touchscreen-autodisabler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.";
 */

#ifndef __TAD_DEVICE_HEADER
#define __TAD_DEVICE_HEADER

#include <stdbool.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/extensions/XInput.h>
#include <X11/extensions/XInput2.h>

#include "tad_def.h"

/* **** Exported variables. ***************************************************/

extern int xi_opcode;
extern int xi_event_code;
extern int xi_error_code;

extern Display*		display;
extern Window		window;
extern XDeviceInfo* devinfo_list;
extern int			devinfo_list_count;

extern XIEventMask   masks[2];
#define mask_watched masks[0]
#define mask_control masks[1]

/* **** Callback Types ********************************************************/

/*
 * WatchCallback:
 * @event:		event structure.
 * @user_data:  user data.
 * 
 * callback type for processing watched events.
 * 
 * Responsiblilty: Check and process event.
 */
typedef		void (*WatchCallback)		(TadEvent		event,
				                         void*			user_data);

void		tad_device_init				(void);
void		tad_device_fini				(void);

void		tad_device_set_watch		(const char*	watched,
				                         const char*	control);

void		tad_device_watch			(WatchCallback  callback,
					                     void*			user_data);

void		tad_mask_set				(XIEventMask*   mask,
						                 XID			device_id,
						                 int			first_eventtype, ...);

XID			tad_device_id_from_args		(const char*	name);

void		tad_device_set_enabled		(XIEventMask*   mask,
		                        		 bool			enabled);

TadEvent	tad_device_event_transform  (XIDeviceEvent* event);

#endif  //TAD_DEVICE_HEADER